@@ -3,6 +3,7 @@
 # Copyright (c) 2005-2006, Bob Ippolito
 # Copyright (c) 2007, Linden Research, Inc.
 # Copyright (c) 2008, Donovan Preston
+# Copyright (c) 2009-2010, Denis Bilenko
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -22,12 +23,14 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-import socket
 import sys
 from code import InteractiveConsole
 
+from gevent import socket
 from gevent.greenlet import Greenlet
-from gevent import core
+from gevent.server import StreamServer
+
+__all__ = ['BackdoorServer']
 
 try:
     sys.ps1
@@ -40,32 +43,14 @@ except AttributeError:
 
 
 class SocketConsole(Greenlet):
-    def __init__(self, desc, hostport, locals):
-        Greenlet.__init__(self)
-        self.hostport = hostport
-        self.locals = locals
-        # mangle the socket
-        self.desc = desc
-        readline = desc.readline
-        self.old = {}
-        self.fixups = {
-            'softspace': 0,
-            'isatty': lambda: True,
-            'flush': lambda: None,
-            'readline': lambda *a: readline(*a).replace('\r\n', '\n'),
-        }
-        for key, value in self.fixups.iteritems():
-            if hasattr(desc, key):
-                self.old[key] = getattr(desc, key)
-            setattr(desc, key, value)
 
-    def _run(self):
-        try:
-            console = InteractiveConsole(self.locals)
-            console.interact()
-        finally:
-            self.switch_out()
-            self.finalize()
+    def __init__(self, locals, conn):
+        Greenlet.__init__(self)
+        self.locals = locals
+        self.desc = _fileobject(conn)
+
+    def finalize(self):
+        self.desc = None
 
     def switch(self, *args, **kw):
         self.saved = sys.stdin, sys.stderr, sys.stdout
@@ -75,48 +60,27 @@ class SocketConsole(Greenlet):
     def switch_out(self):
         sys.stdin, sys.stderr, sys.stdout = self.saved
 
-    def finalize(self):
-        # restore the state of the socket
-        for key in self.fixups:
-            try:
-                value = self.old[key]
-            except KeyError:
-                delattr(self.desc, key)
-            else:
-                setattr(self.desc, key, value)
-        self.fixups.clear()
-        self.old.clear()
-        self.desc = None
-        print "backdoor closed to %s:%s" % self.hostport
-
-
-def backdoor_server(server, locals=None):
-    print "backdoor listening on %s:%s" % server.getsockname()
-    try:
+    def _run(self):
         try:
-            while True:
-                (conn, (host, port)) = server.accept()
-                print "backdoor connected to %s:%s" % (host, port)
-                fl = _fileobject(conn.dup(), "rw", bufsize=1)
-                greenlet = SocketConsole(fl, (host, port), locals)
-                core.active_event(greenlet.switch)
-        except socket.error, e:
-            # Broken pipe means it was shutdown
-            if e[0] != 32:
-                raise
-    finally:
-        server.close()
+            try:
+                console = InteractiveConsole(self.locals)
+                console.interact()
+            except SystemExit:  # raised by quit()
+                sys.exc_clear()
+        finally:
+            self.switch_out()
+            self.finalize()
 
 
-def backdoor((conn, addr), locals=None):
-    """ Use this with tcp_server like so:
-        tcp_server(tcp_listener(('127.0.0.1', 9000)), backdoor.backdoor, {})
-    """
-    host, port = addr
-    print "backdoor to %s:%s" % (host, port)
-    fl = _fileobject(conn.dup(), "rw", bufsize=1)
-    greenlet = SocketConsole(fl, (host, port), locals)
-    core.active_event(greenlet.switch)
+class BackdoorServer(StreamServer):
+
+    def __init__(self, listener, locals=None, **server_args):
+        StreamServer.__init__(self, listener, spawn=None, **server_args)
+        self.locals = locals
+        # QQQ passing pool instance as 'spawn' is not possible; should it be fixed?
+
+    def handle(self, conn, address):
+        SocketConsole.spawn(self.locals, conn)
 
 
 class _fileobject(socket._fileobject):
@@ -124,11 +88,17 @@ class _fileobject(socket._fileobject):
     def write(self, data):
         self._sock.sendall(data)
 
+    def isatty(self):
+        return True
+
+    def flush(self):
+        pass
+
+    def readline(self, *a):
+        return socket._fileobject.readline(self, *a).replace("\r\n", "\n")
 
 if __name__ == '__main__':
     if not sys.argv[1:]:
         print 'USAGE: %s PORT' % sys.argv[0]
     else:
-        from gevent.socket import tcp_server, tcp_listener
-        tcp_server(tcp_listener(('127.0.0.1', int(sys.argv[1]))), backdoor, {})
-
+        BackdoorServer(('127.0.0.1', int(sys.argv[1]))).serve_forever()
