@@ -22,8 +22,17 @@
 """This test checks that socket instances (not gevent.socket.socket but underlying sockets)
 are not leaked by the hub.
 """
-from gevent import monkey
-monkey.patch_all()
+from _socket import socket
+
+
+class Socket(socket):
+    "Something we can have a weakref to"
+
+import _socket
+_socket.socket = Socket
+
+from gevent import monkey; monkey.patch_all()
+
 #import sys
 import greentest
 from pprint import pformat
@@ -32,20 +41,12 @@ from time import sleep
 import weakref
 import gc
 
-from _socket import socket
-
-class Socket(socket):
-    "Something we can have a weakref to"
-
-import _socket
-_socket.socket = Socket
-
 import socket
 socket._realsocket = Socket
 
-
 SOCKET_TIMEOUT = 0.1
 
+
 def init_server():
     s = socket.socket()
     s.settimeout(SOCKET_TIMEOUT)
@@ -54,6 +55,7 @@ def init_server():
     s.listen(5)
     return s
 
+
 def handle_request(s, raise_on_timeout):
     try:
         conn, address = s.accept()
@@ -64,13 +66,14 @@ def handle_request(s, raise_on_timeout):
             return
     #print 'handle_request - accepted'
     res = conn.recv(100)
-    assert res == 'hello', `res`
+    assert res == 'hello', repr(res)
     #print 'handle_request - recvd %r' % res
     res = conn.send('bye')
     #print 'handle_request - sent %r' % res
     #print 'handle_request - conn refcount: %s' % sys.getrefcount(conn)
     #conn.close()
 
+
 def make_request(port):
     #print 'make_request'
     s = socket.socket()
@@ -79,20 +82,22 @@ def make_request(port):
     res = s.send('hello')
     #print 'make_request - sent %s' % res
     res = s.recv(100)
-    assert res == 'bye', `res`
+    assert res == 'bye', repr(res)
     #print 'make_request - recvd %r' % res
     #s.close()
 
+
 def run_interaction(run_client):
     s = init_server()
     start_new_thread(handle_request, (s, run_client))
     if run_client:
         port = s.getsockname()[1]
         start_new_thread(make_request, (port, ))
-    sleep(0.1+SOCKET_TIMEOUT)
-    #print sys.getrefcount(s.fd)
+    sleep(0.1 + SOCKET_TIMEOUT)
+    #print sys.getrefcount(s._sock)
     #s.close()
-    return weakref.ref(s.fd) # XXX OR _sock depending on whether monkey patching is enabled. rename GreenSocket.fd to GreenSocket._sock ?
+    return weakref.ref(s._sock)
+
 
 def run_and_check(run_client):
     w = run_interaction(run_client=run_client)
@@ -104,6 +109,7 @@ def run_and_check(run_client):
                 print '-', pformat(y)
         raise AssertionError('server should be dead by now')
 
+
 class test(greentest.TestCase):
 
     def test_clean_exit(self):
@@ -114,6 +120,6 @@ class test(greentest.TestCase):
         run_and_check(False)
         run_and_check(False)
 
-if __name__=='__main__':
-    greentest.main()
 
+if __name__ == '__main__':
+    greentest.main()
